@@ -4,22 +4,27 @@ Attributes:
     PARAM_MAPPINGS: The type to Param mappings, used for params to init a
         parameter. Managed by `register_param`
 """
-# pylint: disable=too-many-lines
 import ast
-import re
 import json
-from typing import (Optional, List, Any, Callable, Type,
-                    Dict, Set, Tuple, Union, Iterator)
+import re
 from itertools import product
 from pathlib import Path
+from typing import Any, Callable, Dict, Iterator, List, Set, Tuple, Type, Union
+
 from diot import OrderedDiot
-from .utils import cast_to, parse_type, logger, TYPE_NAMES, Namespace
-from .defaults import POSITIONAL, ARGUMENT_REQUIRED
+
 from .completer import CompleterParam
-from .exceptions import (PyParamException, PyParamValueError,
-                         PyParamTypeError, PyParamNameError)
+from .defaults import ARGUMENT_REQUIRED, POSITIONAL
+from .exceptions import (
+    PyParamException,
+    PyParamNameError,
+    PyParamTypeError,
+    PyParamValueError,
+)
+from .utils import TYPE_NAMES, Namespace, cast_to, logger, parse_type
+
+PARAM_MAPPINGS: Dict[str, Type["Param"]] = {}
 
-PARAM_MAPPINGS: Dict[str, Type['Param']] = {}
 
 class Param(CompleterParam):
     """Base class for parameter
@@ -65,45 +70,46 @@ class Param(CompleterParam):
         _value_cached: The cached value calculated from the stack
         _kwargs: other kwargs
     """
-    # pylint: disable=too-many-instance-attributes
-    type: Optional[str] = None
+
+    type: str = None
     type_aliases: List[str] = []
 
     @classmethod
     def on_register(cls):
         """Opens opportunity to do something when a parameter is registered"""
 
-    def __init__(self,
-                 names: List[str],
-                 default: Any,
-                 desc: Optional[List[str]],
-                 prefix: str = 'auto',
-                 show: bool = True,
-                 required: bool = False,
-                 subtype: Optional[bool] = None,
-                 type_frozen: bool = True,
-                 callback: Optional[Callable] = None,
-                 complete_callback: Optional[Callable] = None,
-                 argname_shorten: bool = True,
-                 **kwargs: Dict[str, Any]):
-        # pylint: disable=too-many-arguments
+    def __init__(
+        self,
+        names: Union[str, List[str]],
+        default: Any,
+        desc: List[str],
+        prefix: str = "auto",
+        show: bool = True,
+        required: bool = False,
+        subtype: Union[str, bool] = None,
+        type_frozen: bool = True,
+        callback: Callable = None,
+        complete_callback: Callable = None,
+        argname_shorten: bool = True,
+        **kwargs: Dict[str, Any],
+    ):
         """Constructor"""
-        self.names: List[str] = names
-        self.default: Any = default
+        self.names = names
+        self.default = default
         self.prefix: str = prefix
         self.show: bool = show
         self.required: bool = required
-        self.subtype: Optional[str] = subtype
+        self.subtype = subtype
         self.type_frozen: bool = type_frozen
-        self.callback: Optional[Callable] = callback
-        self.complete_callback: Optional[Callable] = complete_callback
+        self.callback: Callable = callback
+        self.complete_callback: Callable = complete_callback
         self.argname_shorten: bool = argname_shorten
         self.hit: bool = False
         self.is_help: bool = False
-        self.ns_param: Optional["ParamNamespace"] = None
-        self._desc: List[str] = desc or ['No description.']
+        self.ns_param: "ParamNamespace" = None
+        self._desc: List[str] = desc or ["No description."]
         self._stack: List[Any] = []
-        self._value_cached: Optional[Any] = None
+        self._value_cached: Any = None
         self._kwargs: Dict[str, Any] = kwargs
 
         # check if I am under a namespace
@@ -116,7 +122,7 @@ class Param(CompleterParam):
         namespaces: List[Set[str]] = []
         terminals: Set[str] = set()
         for name in self.names:
-            parts: List[str] = name.split('.')
+            parts: List[str] = name.split(".")
             nparts.add(len(parts))
             if len(nparts) > 1:
                 raise PyParamNameError(
@@ -137,13 +143,14 @@ class Param(CompleterParam):
         Returns:
             Name with prefix added
         """
-        name_to_check: str = name.split('.', 1)[0]
-        if self.prefix == 'auto':
+        name_to_check: str = name.split(".", 1)[0]
+        if self.prefix == "auto":
             return f"-{name}" if len(name_to_check) <= 1 else f"--{name}"
         return f"{self.prefix}{name}"
 
-    def namespaces(self,
-                   index: Union[int, str] = 'len') -> Union[List[str], int]:
+    def namespaces(
+        self, index: Union[int, str] = "len"
+    ) -> Union[List[str], int]:
         """Get the namespaces at the given index or number of namespaces
 
         Args:
@@ -152,10 +159,10 @@ class Param(CompleterParam):
         Returns:
             The length of the namespaces or the namespaces at index.
         """
-        if index == 'len':
+        if index == "len":
             return len(self._namespaces)
         try:
-            return self._namespaces[index]
+            return self._namespaces[index]  # type: ignore
         except IndexError:
             return []
 
@@ -169,8 +176,11 @@ class Param(CompleterParam):
         Returns:
             The names with full combinations of namespaces and terminals
         """
-        self.names = ['.'.join(prod)
-                      for prod in product(*self._namespaces, self.terminals)]
+        self.names = [
+            ".".join(prod)
+            for prod in product(*self._namespaces, self.terminals)
+        ]
+        return self.names
 
     @property
     def is_positional(self) -> bool:
@@ -190,7 +200,7 @@ class Param(CompleterParam):
         if self.hit:
             logger.warning("No value provided for argument %r", self.namestr())
 
-    def overwrite_type(self, param_type: Optional[str]) -> Type['Param']:
+    def overwrite_type(self, param_type: str) -> "Param":
         """Try to overwrite the type
 
         Only when param_type is not None and it's different from mine
@@ -207,10 +217,15 @@ class Param(CompleterParam):
             return self
 
         if self.type_frozen:
-            raise PyParamTypeError(f"Type of argument {self.namestr()!r} "
-                                   "is not overwritable")
-        logger.warning("Type changed from %r to %r for argument %r",
-                       self.typestr(), param_type, self.namestr())
+            raise PyParamTypeError(
+                f"Type of argument {self.namestr()!r} " "is not overwritable"
+            )
+        logger.warning(
+            "Type changed from %r to %r for argument %r",
+            self.typestr(),
+            param_type,
+            self.namestr(),
+        )
         return self.to(param_type)
 
     def consume(self, value: Any) -> bool:
@@ -231,8 +246,11 @@ class Param(CompleterParam):
     def desc(self) -> List[str]:
         """The formatted description using attributes and _kwargs"""
         format_data: dict = {
-            key: val for key, val in self.__dict__.items()
-            if (not key.startswith('_') and key != 'desc' and not callable(val))
+            key: val
+            for key, val in self.__dict__.items()
+            if (
+                not key.startswith("_") and key != "desc" and not callable(val)
+            )
         }
         format_data.update(self._kwargs)
         ret: List[str] = []
@@ -241,13 +259,13 @@ class Param(CompleterParam):
                 descr = descr.format(**format_data)
             except KeyError as kerr:
                 raise PyParamException(
-                    f'Description of {self.namestr()!r} is formatting '
-                    'using kwargs from contructor. \n'
-                    'If you have curly braces in it, which is not intended '
-                    'for formatting, please escape it by replacing with '
-                    '`{{` or `}}`:\n'
-                    f'- desc: {descr}\n'
-                    f'- key : {{... {str(kerr)[1:-1]} ...}}'
+                    f"Description of {self.namestr()!r} is formatting "
+                    "using kwargs from contructor. \n"
+                    "If you have curly braces in it, which is not intended "
+                    "for formatting, please escape it by replacing with "
+                    "`{{` or `}}`:\n"
+                    f"- desc: {descr}\n"
+                    f"- key : {{... {str(kerr)[1:-1]} ...}}"
                 ) from None
             else:
                 ret.append(descr)
@@ -267,8 +285,9 @@ class Param(CompleterParam):
         Returns:
             The shortest/longest name of the parameter
         """
-        name: str = list(sorted(self.names,
-                                key=len))[0 if 'short' in which else -1]
+        name: str = list(sorted(self.names, key=len))[
+            0 if "short" in which else -1
+        ]
         return name if not with_prefix else self._prefix_name(name)
 
     def namestr(self, sep: str = ", ", with_prefix: bool = True) -> str:
@@ -280,9 +299,14 @@ class Param(CompleterParam):
         Returns:
             the connected names
         """
-        names: list = ['POSITIONAL' if name == POSITIONAL
-                       else self._prefix_name(name) if with_prefix
-                       else name for name in sorted(self.names, key=len)]
+        names: list = [
+            "POSITIONAL"
+            if name == POSITIONAL
+            else self._prefix_name(name)
+            if with_prefix
+            else name
+            for name in sorted(self.names, key=len)
+        ]
         return sep.join(names)
 
     def typestr(self) -> str:
@@ -301,7 +325,7 @@ class Param(CompleterParam):
             the string representation of the parameter in the default usage
         """
         # * makes sure it's not wrapped
-        ret: str = self.name('long') + "*"
+        ret: str = self.name("long") + "*"
         ret += self.typestr().upper() if self.type_frozen else self.typestr()
         return ret
 
@@ -313,19 +337,23 @@ class Param(CompleterParam):
             the string representation of the parameter names and types
                 in the optname section in help page
         """
-        typestr: str = (self.typestr().upper()
-                        if self.type_frozen else self.typestr())
+        typestr: str = (
+            self.typestr().upper() if self.type_frozen else self.typestr()
+        )
         if not self.ns_param or not self.argname_shorten:
             # * makes sure it's not wrapped
-            return (self.namestr() if self.is_help
-                    else f"{self.namestr()}*<{typestr}>")
+            return (
+                self.namestr()
+                if self.is_help
+                else f"{self.namestr()}*<{typestr}>"
+            )
 
         ret: List[str] = []
         for term in sorted(self.terminals, key=len):
             ret.append(f"~<ns>.{term}")
         return ", ".join(ret) + f"*<{typestr}>"
 
-    def to(self, to_type: str) -> Type['Param']:
+    def to(self, to_type: str) -> "Param":
         """Generate a different type of parameter using current settings
 
         Args:
@@ -334,10 +362,9 @@ class Param(CompleterParam):
         Returns:
             the generated parameter with different type
         """
-        # Type: Optional[str], Optional[str]
         main_type, sub_type = parse_type(to_type)
         klass: Callable = PARAM_MAPPINGS[main_type]
-        param: Type['Param'] = klass(
+        param: "Param" = klass(
             names=self.names,
             default=None,
             desc=self.desc,
@@ -348,12 +375,12 @@ class Param(CompleterParam):
             callback=self.callback,
             complete_callback=self.complete_callback,
             argname_shorten=self.argname_shorten,
-            **self._kwargs
+            **self._kwargs,
         )
         param.ns_param = self.ns_param
         return param
 
-    def copy(self) -> Type['Param']:
+    def copy(self) -> "Param":
         """Copy a parameter so that it can be reused.
 
         Returns:
@@ -371,7 +398,7 @@ class Param(CompleterParam):
             type_frozen=self.type_frozen,
             complete_callback=self.complete_callback,
             argname_shorten=self.argname_shorten,
-            **self._kwargs
+            **self._kwargs,
         )
 
     @property
@@ -382,11 +409,14 @@ class Param(CompleterParam):
             the default group name
         """
         ret: str = "REQUIRED OPTIONS" if self.required else "OPTIONAL OPTIONS"
-        return (ret if not self.ns_param
-                else f"{ret} UNDER {self.ns_param.name('long')}")
+        return (
+            ret
+            if not self.ns_param
+            else f"{ret} UNDER {self.ns_param.name('long')}"
+        )
 
     @property
-    def desc_with_default(self) -> Optional[List[str]]:
+    def desc_with_default(self) -> List[str]:
         """If default is not specified in desc, just to add with the default
         value
 
@@ -396,14 +426,15 @@ class Param(CompleterParam):
         if self.is_help:
             return self.desc
 
-        if (self.required or (
-                self.desc and
-                any("Default:" in desc or "DEFAULT:" in desc
-                    for desc in self.desc)
-        )):
+        if self.required or (
+            self.desc
+            and any(
+                "Default:" in desc or "DEFAULT:" in desc for desc in self.desc
+            )
+        ):
             return None if self.desc is None else self.desc[:]
 
-        desc: List[str] = self.desc[:] if self.desc else ['']
+        desc: List[str] = self.desc[:] if self.desc else [""]
         if desc[0] and not desc[0][-1:].isspace():
             desc[0] += " "
 
@@ -420,7 +451,8 @@ class Param(CompleterParam):
         if self.hit is True and self._stack:
             logger.warning(
                 "Previous value of argument %r is overwritten with %r.",
-                self.namestr(), item
+                self.namestr(),
+                item,
             )
             self._stack = []
 
@@ -430,8 +462,10 @@ class Param(CompleterParam):
         self.hit = False
 
     def __repr__(self) -> str:
-        return (f'<{self.__class__.__name__}({self.namestr()} :: '
-                f'{self.typestr()}) @ {id(self)}>')
+        return (
+            f"<{self.__class__.__name__}({self.namestr()} :: "
+            f"{self.typestr()}) @ {id(self)}>"
+        )
 
     def _value(self) -> Any:
         """Get the organized value of this parameter
@@ -480,7 +514,7 @@ class Param(CompleterParam):
         except TypeError as terr:
             # len() takes exactly one argument (2 given)
             # <lambda>() takes 1 positional argument but 2 were given
-            if not re.search(r'takes .+ argument .+ given', str(terr)):
+            if not re.search(r"takes .+ argument .+ given", str(terr)):
                 raise
             val = self.callback(self.value)
 
@@ -488,18 +522,22 @@ class Param(CompleterParam):
             raise PyParamTypeError(str(val))
         return val
 
+
 class ParamAuto(Param):
     """An auto parameter whose value is automatically casted"""
-    type: str = 'auto'
+
+    type: str = "auto"
 
     def _value(self) -> Any:
         """Cast value automatically"""
-        return cast_to(super()._value(), 'auto')
+        return cast_to(super()._value(), "auto")
+
 
 class ParamInt(Param):
     """An int parameter whose value is automatically casted into an int"""
-    type: str = 'int'
-    type_aliases: List[str] = ['i']
+
+    type: str = "int"
+    type_aliases: List[str] = ["i"]
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -512,10 +550,12 @@ class ParamInt(Param):
         except ValueError as verr:
             raise PyParamValueError(str(verr)) from None
 
+
 class ParamFloat(Param):
     """A float parameter whose value is automatically casted into a float"""
-    type: str = 'float'
-    type_aliases: List[str] = ['f']
+
+    type: str = "float"
+    type_aliases: List[str] = ["f"]
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -528,26 +568,30 @@ class ParamFloat(Param):
         except ValueError as verr:
             raise PyParamValueError(str(verr)) from None
 
+
 class ParamStr(Param):
     """A str parameter whose value is automatically casted into a str"""
-    type: str = 'str'
-    type_aliases: List[str] = ['s']
+
+    type: str = "str"
+    type_aliases: List[str] = ["s"]
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         # self.default = self.default or ''
 
+
 class ParamBool(Param):
     """A bool parameter whose value is automatically casted into a bool"""
-    type: str = 'bool'
-    type_aliases: List[str] = ['b', 'flag']
+
+    type: str = "bool"
+    type_aliases: List[str] = ["b", "flag"]
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        #self.default = self.default or False
+        # self.default = self.default or False
         if self.default is not None:
             try:
-                self.default = cast_to(str(self.default), 'bool')
+                self.default = cast_to(str(self.default), "bool")
             except (PyParamValueError, PyParamTypeError):
                 raise PyParamValueError(
                     "Default value of a count argument must be a bool value or "
@@ -561,7 +605,7 @@ class ParamBool(Param):
         Returns:
             the string representation of the parameter in the default usage
         """
-        return self.name('long')
+        return self.name("long")
 
     def optstr(self) -> str:
         """Get the string representation of the parameter names and types
@@ -573,8 +617,9 @@ class ParamBool(Param):
         """
         if self.is_help:
             return self.namestr()
-        typestr: str = (self.typestr().upper()
-                        if self.type_frozen else self.typestr())
+        typestr: str = (
+            self.typestr().upper() if self.type_frozen else self.typestr()
+        )
 
         if not self.ns_param or not self.argname_shorten:
             # * makes sure it's not wrapped'
@@ -587,7 +632,7 @@ class ParamBool(Param):
 
     def close(self) -> None:
         if self.hit is True:
-            self.push('true')
+            self.push("true")
 
     def consume(self, value: str) -> bool:
         """Should I consume given value?"""
@@ -595,52 +640,54 @@ class ParamBool(Param):
             return False
 
         try:
-            cast_to(value, 'bool')
+            cast_to(value, "bool")
         except (PyParamValueError, PyParamTypeError):
-            return False # cannot cast, don't consume
+            return False  # cannot cast, don't consume
         else:
             self.push(value)
             return True
 
-    def _value(self) -> Optional[bool]:
+    def _value(self) -> bool:
         if not self._stack:
             return self.default
 
         val = self._stack[-1][0]
-        ret = None if val is None else cast_to(val, 'bool')
+        ret = None if val is None else cast_to(val, "bool")
         self._stack = []
         return ret
 
     def complete_value(
-            self,
-            current: str,
-            prefix: str = ''
-    ) -> Optional[Union[
-        str, Iterator[Tuple[str]],
-        Iterator[Tuple[str, str]], Iterator[Tuple[str, str, str]]
-    ]]:
+        self, current: str, prefix: str = ""
+    ) -> Union[
+        str,
+        Iterator[Tuple[str]],
+        Iterator[Tuple[str, str]],
+        Iterator[Tuple[str, str, str]],
+    ]:
         """Get the completion candidates for the current parameter"""
         if self.is_help:
-            return ''
+            return ""
         if callable(self.complete_callback):
             return super().complete_value(current, prefix)
         if current:
-            trues: List[str] = ('True', 'true', 'TRUE', '1')
-            falses: List[str] = ('False', 'false', 'FALSE', '0')
+            trues: List[str] = ("True", "true", "TRUE", "1")
+            falses: List[str] = ("False", "false", "FALSE", "0")
             ret: List[Tuple[str, str, str]] = []
             for cand in trues:
                 if cand.startswith(current):
-                    ret.append((f'{prefix}{cand}', 'plain', 'Value True'))
+                    ret.append((f"{prefix}{cand}", "plain", "Value True"))
 
             for cand in falses:
                 if cand.startswith(current):
-                    ret.append((f'{prefix}{cand}', 'plain', 'Value False'))
+                    ret.append((f"{prefix}{cand}", "plain", "Value False"))
             return ret
-        return ''
+        return ""
+
 
 class ParamCount(Param):
     """A bool parameter whose value is automatically casted into a bool"""
-    type: str = 'count'
+
+    type: str = "count"
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -651,20 +698,18 @@ class ParamCount(Param):
                 "Default value of a count argument must be 0"
             )
 
-        if len(self.name('short', with_prefix=False)) != 1:
-            raise PyParamValueError(
-                "Count argument must have a short name."
-            )
+        if len(self.name("short", with_prefix=False)) != 1:
+            raise PyParamValueError("Count argument must have a short name.")
 
-        self._kwargs.setdefault('max', 0) # 0: no max
-        if not isinstance(self._kwargs['max'], int) or self._kwargs['max'] < 0:
+        self._kwargs.setdefault("max", 0)  # 0: no max
+        if not isinstance(self._kwargs["max"], int) or self._kwargs["max"] < 0:
             raise PyParamValueError(
                 "Argument 'max' for count argument must be a positive integer"
             )
 
     def close(self) -> None:
         if self.hit is True:
-            self.push('1')
+            self.push("1")
 
     def consume(self, value: str) -> bool:
         """Should I consume given parameter?"""
@@ -689,11 +734,15 @@ class ParamCount(Param):
                     retval = len(val) + 1
                     break
         if retval is None:
-            raise PyParamValueError("Expect repeated short names or "
-                                    "an integer as count argument value.")
-        if self._kwargs['max'] and retval > self._kwargs['max']:
-            raise PyParamValueError(f"{retval} is greater than "
-                                    f"the max of {self._kwargs['max']}.")
+            raise PyParamValueError(
+                "Expect repeated short names or "
+                "an integer as count argument value."
+            )
+        if self._kwargs["max"] and retval > self._kwargs["max"]:
+            raise PyParamValueError(
+                f"{retval} is greater than "
+                f"the max of {self._kwargs['max']}."
+            )
         return retval
 
     def complete_name(self, current: str) -> Iterator[Tuple[str, str]]:
@@ -711,12 +760,15 @@ class ParamCount(Param):
                 continue
             # check the max
             value: int = len(current) - 1
-            if self._kwargs['max'] and value >= self._kwargs['max']:
+            if self._kwargs["max"] and value >= self._kwargs["max"]:
                 # already max'ed, no further completion
                 continue
 
-            ncompletes: int = (min(self._kwargs['max'] - value, 2)
-                               if self._kwargs['max'] else 2)
+            ncompletes: int = (
+                min(self._kwargs["max"] - value, 2)
+                if self._kwargs["max"]
+                else 2
+            )
             for i in range(ncompletes):
                 yield current + name * (i + 1), self.desc[0].splitlines()[0]
             break
@@ -724,24 +776,21 @@ class ParamCount(Param):
         else:
             yield from super().complete_name(current)
 
+
 class ParamPath(Param):
     """A path parameter whose value is automatically casted into a pathlib.Path
     """
-    type: str = 'path'
-    type_aliases: List[str] = ['p', 'file']
+
+    type: str = "path"
+    type_aliases: List[str] = ["p", "file"]
 
     def _value(self) -> Path:
-        val: Optional[Path] = super()._value()
+        val: Path = super()._value()
         return None if val is None else Path(val)
 
     def complete_value(
-            self,
-            current: str,
-            prefix: str = ''
-    ) -> Optional[Union[
-        str, Iterator[Tuple[str]],
-        Iterator[Tuple[str, str]], Iterator[Tuple[str, str, str]]
-    ]]:
+        self, current: str, prefix: str = ""
+    ) -> Union[str, Iterator[Tuple[str, ...]]]:
         """Generate file paths with given current prefix
         as completion candidates
 
@@ -750,7 +799,8 @@ class ParamPath(Param):
         """
         if callable(self.complete_callback):
             return super().complete_value(current, prefix)
-        return [(current, 'file', prefix)]
+        return [(current, "file", prefix)]
+
 
 class ParamDir(ParamPath):
     """Subclass of ParamPath.
@@ -759,17 +809,13 @@ class ParamDir(ParamPath):
     for completions. The completion items for this param will only give
     directories instead of all paths
     """
-    type: str = 'dir'
+
+    type: str = "dir"
     type_aliases: List[str] = []
 
     def complete_value(
-            self,
-            current: str,
-            prefix: str = ''
-    ) -> Optional[Union[
-        str, Iterator[Tuple[str]],
-        Iterator[Tuple[str, str]], Iterator[Tuple[str, str, str]]
-    ]]:
+        self, current: str, prefix: str = ""
+    ) -> Union[str, Iterator[Tuple[str, ...]]]:
         """Generate dir paths with given current prefix as completion candidates
 
         Args:
@@ -777,35 +823,41 @@ class ParamDir(ParamPath):
         """
         if callable(self.complete_callback):
             return super().complete_value(current, prefix)
-        return [(current, 'dir', prefix)]
+        return [(current, "dir", prefix)]
+
 
 class ParamPy(Param):
     """A parameter whose value will be ast.literal_eval'ed"""
-    type: str = 'py'
+
+    type: str = "py"
 
     def _value(self) -> Any:
         return ast.literal_eval(str(super()._value()))
 
+
 class ParamJson(Param):
     """A parameter whose value will be parsed as json"""
-    type: str = 'json'
-    type_aliases: List[str] = ['j']
+
+    type: str = "json"
+    type_aliases: List[str] = ["j"]
 
     def _value(self) -> Any:
         val: Any = super()._value()
         return None if val is None else json.loads(str(val))
 
+
 class ParamList(Param):
     """A parameter whose value is a list"""
-    type: str = 'list'
-    type_aliases: List[str] = ['l', 'a', 'array']
+
+    type: str = "list"
+    type_aliases: List[str] = ["l", "a", "array"]
     complete_relapse: bool = True
 
     @classmethod
     def on_register(cls):
         """Also register reset type"""
-        name = 'reset'
-        aliases = ['r']
+        name = "reset"
+        aliases = ["r"]
         all_names = [name] + aliases
 
         for nam in all_names:
@@ -816,9 +868,9 @@ class ParamList(Param):
         self.default = self.default or []
         self._stack.append(self.default)
 
-    def overwrite_type(self, param_type: Optional[str]) -> Type['Param']:
+    def overwrite_type(self, param_type: str) -> "Param":
         """Deal with when param_type is reset"""
-        if param_type == 'reset':
+        if param_type == "reset":
             self._stack = []
             return self
         return super().overwrite_type(param_type)
@@ -837,29 +889,36 @@ class ParamList(Param):
 
     def _value(self) -> List[Any]:
         """Get the value a list parameter"""
-        ret = [cast_to(val, self.subtype) for sublist in self._stack
-               for val in sublist]
+        ret = [
+            cast_to(val, self.subtype)
+            for sublist in self._stack
+            for val in sublist
+        ]
 
         if self.required and not ret:
             raise PyParamValueError(ARGUMENT_REQUIRED)
         self._stack = []
         return ret
 
+
 class ParamChoice(Param):
     """A bool parameter whose value is automatically casted into a bool"""
-    type: str = 'choice'
-    type_aliases: List[str] = ['c']
+
+    type: str = "choice"
+    type_aliases: List[str] = ["c"]
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
-        if 'choices' not in self._kwargs:
-            raise PyParamValueError("Argument 'choices' is required "
-                                    "for ParamChoice.")
+        if "choices" not in self._kwargs:
+            raise PyParamValueError(
+                "Argument 'choices' is required " "for ParamChoice."
+            )
 
-        if not isinstance(self._kwargs['choices'], (list, tuple)):
-            raise PyParamValueError("Argument 'choices' must be "
-                                    "a list or a tuple.")
+        if not isinstance(self._kwargs["choices"], (list, tuple)):
+            raise PyParamValueError(
+                "Argument 'choices' must be " "a list or a tuple."
+            )
 
         # if not self.required and self.default is None:
         #     self.default = self._kwargs['choices'][0]
@@ -867,7 +926,7 @@ class ParamChoice(Param):
     def _value(self) -> Any:
         val = super()._value()
 
-        if val is not None and val not in self._kwargs['choices']:
+        if val is not None and val not in self._kwargs["choices"]:
             raise PyParamValueError(
                 f"{val} is not one of {self._kwargs['choices']}"
             )
@@ -875,13 +934,8 @@ class ParamChoice(Param):
         return val
 
     def complete_value(
-            self,
-            current: str,
-            prefix: str = ''
-    ) -> Optional[Union[
-        str, Iterator[Tuple[str]],
-        Iterator[Tuple[str, str]], Iterator[Tuple[str, str, str]]
-    ]]:
+        self, current: str, prefix: str = ""
+    ) -> Union[str, Iterator[Tuple[str, ...]]]:
         """Generate choices with given current prefix as completion candidates
 
         Args:
@@ -891,11 +945,12 @@ class ParamChoice(Param):
             return super().complete_value(current, prefix)
 
         ret: Iterator[Tuple[str]] = []
-        for choice in self._kwargs['choices']:
+        for choice in self._kwargs["choices"]:
             if choice.startswith(current):
-                ret.append((f"{prefix}{choice}", ))
+                ret.append((f"{prefix}{choice}",))
         return ret
 
+
 class ParamNamespace(Param):
     """A pseudo parameter serving as a namespace for parameters under it
 
@@ -904,33 +959,39 @@ class ParamNamespace(Param):
     prog --namespace.arg1 1 --namespace.arg2 2
     ```
     """
-    type: str = 'ns'
-    type_aliases: List[str] = ['namespace']
+
+    type: str = "ns"
+    type_aliases: List[str] = ["namespace"]
 
     def __init__(self, *args, **kwargs):
-        kwargs['desc'] = kwargs.get('desc') or [
+        kwargs["desc"] = kwargs.get("desc") or [
             "Works as a namespace for other arguments.",
-            "Never pass a value directly to it."
+            "Never pass a value directly to it.",
         ]
-        kwargs['default'] = None
+        kwargs["default"] = None
         super().__init__(*args, **kwargs)
-        self._stack = OrderedDiot() # for my decendents
+        self._stack = OrderedDiot()  # for my decendents
 
     @property
     def default_group(self) -> str:
         """Get the default group of the parameter"""
-        ret: str = ('REQUIRED'
-                    if any(param.required for param in self._stack.values())
-                    else 'OPTIONAL')
-        return (f'{ret} OPTIONS' if not self.ns_param
-                else f'{ret} OPTIONS UNDER {self.ns_param.name("long")}')
+        ret: str = (
+            "REQUIRED"
+            if any(param.required for param in self._stack.values())
+            else "OPTIONAL"
+        )
+        return (
+            f"{ret} OPTIONS"
+            if not self.ns_param
+            else f'{ret} OPTIONS UNDER {self.ns_param.name("long")}'
+        )
 
     @property
-    def desc_with_default(self) -> Optional[List[str]]:
+    def desc_with_default(self) -> List[str]:
         """Namespace parameters do not have a default value"""
         return self.desc[:]
 
-    def decendents(self, show_only=False) -> List[Type['Param']]:
+    def decendents(self, show_only=False) -> List["Param"]:
         """Get all decendents of this namespace parameter
 
         Args:
@@ -939,7 +1000,7 @@ class ParamNamespace(Param):
         Returns:
             The decendents of this namespace parameter
         """
-        ret: List[Type['Param']] = []
+        ret: List["Param"] = []
         ret_append: Callable = ret.append
         for param in self._stack.values():
             # don't skip entire ns parameter
@@ -958,7 +1019,7 @@ class ParamNamespace(Param):
         """Should I consume given parameter?"""
         return False
 
-    def get_param(self, name: str, depth: int = 0) -> Optional[Type["Param"]]:
+    def get_param(self, name: str, depth: int = 0) -> "Param":
         """Get the paraemeter by given name
 
         This parameter is like '-a', and the name can be 'a.b.c.d'
@@ -970,20 +1031,22 @@ class ParamNamespace(Param):
         Returns:
             The parameter we get with the given name
         """
-        parts: List[str] = name.split('.')
+        parts: List[str] = name.split(".")
         if depth < len(parts) - 1:
             part = parts[depth + 1]
             if part not in self._stack:
                 return None
-            if not isinstance(self._stack[part], ParamNamespace):
-                return self._stack[part]
-            return self._stack[part].get_param(name, depth + 1)
+            if not isinstance(
+                self._stack[part],  # type: ignore
+                ParamNamespace,
+            ):
+                return self._stack[part]  # type: ignore
+            return self._stack[part].get_param(name, depth + 1)  # type: ignore
         if name in self.names:
             return self
         return None
 
-    # pylint: disable=arguments-differ
-    def push(self, item: Type["Param"], depth: int = 0) -> None:
+    def push(self, item: "Param", depth: int = 0) -> None:
         """Push the parameter under this namespace.
 
         We are not pushing any values to this namespace, but pushing
@@ -991,28 +1054,34 @@ class ParamNamespace(Param):
         """
         # check if item's namespaces at depth contain only names of this
         # ns parameter
-        if set(item.namespaces(depth)) - set(self.terminals):
+        if set(item.namespaces(depth)) - set(self.terminals):  # type: ignore
             raise PyParamValueError(
                 "Parameter names should only contain namespace names "
                 "that belong to the same namespace parameter."
             )
 
         # set the names with all possible combination
-        item._namespaces[depth] = self.terminals #[:]
+        item._namespaces[depth] = self.terminals  # [:]
         item.full_names()
 
         # check if we have nested namespaces
-        if 0 <= depth < item.namespaces('len') - 1:
+        if 0 <= depth < item.namespaces("len") - 1:  # type: ignore
             # see if sub-ns exists
-            subns_name = item.namespaces(depth + 1)[0]
+            subns_name = item.namespaces(depth + 1)[0]  # type: ignore
             if subns_name in self._stack:
-                subns = self._stack[subns_name]
+                subns = self._stack[subns_name]  # type: ignore
             else:
-                subns = ParamNamespace([
-                    '.'.join(prod) for prod in product(
-                        *(item.namespaces(i) for i in range(depth+2))
-                    )
-                ])
+                subns = ParamNamespace(
+                    [
+                        ".".join(prod)
+                        for prod in product(
+                            *(
+                                item.namespaces(i)  # type: ignore
+                                for i in range(depth + 2)
+                            )
+                        )
+                    ]
+                )
                 subns.ns_param = self
 
                 for name in subns.terminals:
@@ -1021,8 +1090,7 @@ class ParamNamespace(Param):
         else:
             item.ns_param = self
             for term in item.terminals:
-                self._stack[term] = item
-    # pylint: enable=arguments-differ
+                self._stack[term] = item  # type: ignore
 
     def _value(self) -> Namespace:
         val = Namespace()
@@ -1032,15 +1100,16 @@ class ParamNamespace(Param):
                     val[name] = param.value
         return val
 
-    def copy(self) -> Type['Param']:
+    def copy(self) -> "Param":
         """Copy a parameter so that it can be reused.
 
         Returns:
             The copy of the parameter
         """
         copied = super().copy()
-        copied._stack = OrderedDiot([(key, param.copy())
-                                     for key, param in self._stack.items()])
+        copied._stack = OrderedDiot(
+            [(key, param.copy()) for key, param in self._stack.items()]
+        )
         return copied
 
     def apply_callback(self, all_values: Namespace) -> Any:
@@ -1048,7 +1117,9 @@ class ParamNamespace(Param):
         for param_name, param in self._stack.items():
             if param_name not in ns_callback_applied:
                 for name in param.terminals:
-                    ns_callback_applied[name] = param.apply_callback(all_values)
+                    ns_callback_applied[name] = param.apply_callback(
+                        all_values
+                    )
 
         if not callable(self.callback):
             return ns_callback_applied
@@ -1058,7 +1129,7 @@ class ParamNamespace(Param):
         except TypeError as terr:
             # len() takes exactly one argument (2 given)
             # <lambda>() takes 1 positional argument but 2 were given
-            if not re.search(r'takes .+ argument .+ given', str(terr)):
+            if not re.search(r"takes .+ argument .+ given", str(terr)):
                 raise
             val = self.callback(ns_callback_applied)
 
@@ -1066,7 +1137,8 @@ class ParamNamespace(Param):
             raise PyParamTypeError(str(val))
         return val
 
-def register_param(param: Param) -> None:
+
+def register_param(param: Type[Param]) -> None:
     """Register a parameter class
 
     Args:
@@ -1076,14 +1148,28 @@ def register_param(param: Param) -> None:
     """
     for alias in param.type_aliases + [param.type]:
         if alias in TYPE_NAMES:
-            raise PyParamNameError('Type name has already been '
-                                   f'registered: {alias}')
+            raise PyParamNameError(
+                "Type name has already been " f"registered: {alias}"
+            )
         TYPE_NAMES[alias] = param.type
 
     PARAM_MAPPINGS[param.type] = param
     param.on_register()
 
-for param_class in (ParamAuto, ParamInt, ParamStr, ParamFloat, ParamBool,
-                    ParamCount, ParamPath, ParamDir, ParamPy, ParamJson,
-                    ParamList, ParamChoice, ParamNamespace):
+
+for param_class in (
+    ParamAuto,
+    ParamInt,
+    ParamStr,
+    ParamFloat,
+    ParamBool,
+    ParamCount,
+    ParamPath,
+    ParamDir,
+    ParamPy,
+    ParamJson,
+    ParamList,
+    ParamChoice,
+    ParamNamespace,
+):
     register_param(param_class)
